@@ -1,8 +1,8 @@
 /****************** Start of $RCSfile: fileutil.c,v $  ****************
 *
-* $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/fileutil.c,v $
-* $Id: fileutil.c,v 1.8 2005/07/31 15:42:09 alb Exp alb $
-* $Date: 2005/07/31 15:42:09 $
+* $Source: /home/alb/afbackup/afbackup-3.5.2.1/RCS/fileutil.c,v $
+* $Id: fileutil.c,v 1.10 2006/12/12 20:21:10 alb Exp alb $
+* $Date: 2006/12/12 20:21:10 $
 * $Author: alb $
 *
 *
@@ -15,7 +15,7 @@
 #include <conf.h>
 #include <version.h>
 
-  static char * fileversion = "$RCSfile: fileutil.c,v $ $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/fileutil.c,v $ $Id: fileutil.c,v 1.8 2005/07/31 15:42:09 alb Exp alb $ " PACKAGE " " VERSION_STRING;
+  static char * fileversion = "$RCSfile: fileutil.c,v $ $Source: /home/alb/afbackup/afbackup-3.5.2.1/RCS/fileutil.c,v $ $Id: fileutil.c,v 1.10 2006/12/12 20:21:10 alb Exp alb $ " PACKAGE " " VERSION_STRING;
 
 #include <stdio.h>
 #include <string.h>
@@ -320,20 +320,12 @@ read_param_file(
   CLEANUP;
 }
 
-static	UChar	*yesstrings[] = {
-			"1",
-			"yes",
-			"true",
-			"ok",
-			NULL,
-		};
-
 static Int32
 get_entry(
   ParamFileEntry	*entry,
   UChar			*line)
 {
-  UChar		*cptr, **cpptr;
+  UChar		*cptr;
   Uns32		num_values;
   void		*values = NULL;
   double	indouble;
@@ -470,12 +462,7 @@ get_entry(
 
 	cptr = first_nospace(cptr);
 	j = first_space(cptr) - cptr;
-	for(cpptr = yesstrings; *cpptr; cpptr++){
-	  if(!strncasecmp(*cpptr, cptr, j)){
-	    *((Flag *) entry->entry_ptr) = YES;
-	    break;
-	  }
-	}
+	*((Flag *) entry->entry_ptr) = is_yes(cptr, j);
 	break;
     case TypeReal64PTR:
 	values = seg_malloc(sizeof(Real64));
@@ -978,7 +965,7 @@ cleanpath(UChar * path)
 
   if(FN_LEADINGDUMMY(path)){
     cptr1 = path;
-    cptr2 = path + 2;
+    cptr2 = path + strlen(FN_CURDIR FN_DIRSEPSTR);
     while(*cptr2)
       *(cptr1++) = *(cptr2++);
 
@@ -1113,6 +1100,9 @@ mkabspath(UChar * path, UChar * retpath)
 {
   UChar	*curpath = NULL, lbuf[256];
 
+  if(!path)
+    return(NULL);
+
   if(FN_ISABSPATH(path)){
     if(retpath){
 	strcpy(retpath, path);
@@ -1123,8 +1113,8 @@ mkabspath(UChar * path, UChar * retpath)
     }
   }
 
-  if(FN_LEADINGDUMMY(path))
-    path += 2;
+  while(FN_LEADINGDUMMY(path))
+    path += strlen(FN_CURDIR FN_DIRSEPSTR);
 
   curpath = get_cwd(lbuf, 256);
 
@@ -2156,7 +2146,7 @@ kfile_proc(UChar * filen, UChar * key, UChar * value, Uns32 flags, Flag insert)
   lockfd = -1;
   if(flags & KFILE_LOCKED){
     lockfd = set_wlock_forced(filen);
-    if(lockfd < 0)
+    if(lockfd < 0 && !(flags & KFILE_OPTLOCK))
 	cleanup_ret(-7);
   }
 
@@ -2283,7 +2273,9 @@ kfile_proc(UChar * filen, UChar * key, UChar * value, Uns32 flags, Flag insert)
  * <key> fields as numerical values and to sort the lines accordingly.
  * If both are set, sorting by numerical value has higher priority than
  * sorting by ASCII. If KFILE_LOCKED is set in <flags>, then the file is
- * locked before modifying it.
+ * locked before modifying it. If KFILE_OPTLOCK is set (optional lock,
+ * includes KFILE_LOCKED), a lock is attempted, but if that fails, the
+ * failure is ignored and processing continues.
  * If <filen> does not exist, it is created.
  *
  * Returns NULL on error.
@@ -2339,8 +2331,8 @@ kfile_get(UChar * filen, UChar * key, Uns32 flags)
 
   lockfd = -1;
   if(flags & KFILE_LOCKED){
-    lockfd = set_wlock_forced(filen);
-    if(lockfd < 0)
+    lockfd = set_rlock_forced(filen);
+    if(lockfd < 0 && !(flags & KFILE_OPTLOCK))
 	return(NULL);
   }
 
@@ -2397,14 +2389,14 @@ kfile_getall(UChar * filename, Int32 * num, Uns32 flags)
   int		lockfd;		/* uninitialized ok */
 
   if(flags & KFILE_LOCKED){
-    lockfd = set_wlock_forced(filename);
-    if(lockfd < 0)
+    lockfd = set_rlock_forced(filename);
+    if(lockfd < 0 && !(flags & KFILE_OPTLOCK))	/* optional lock */
 	return(NULL);
   }
 
   lines = read_asc_file(filename, &num_lines);
 
-  if(flags & KFILE_LOCKED)
+  if((flags & KFILE_LOCKED) && lockfd >= 0)
     close(lockfd);
 
   if(!lines){
@@ -2441,6 +2433,11 @@ kfile_getall(UChar * filename, Int32 * num, Uns32 flags)
   if(num)
     *num = n;
 
+  if(flags & KFILE_SORTN)
+    q_sort(all_pairs, n, sizeof(all_pairs[0]), cmp_KeyValue_bykeyn);
+  else if(flags & KFILE_SORT)
+    q_sort(all_pairs, n, sizeof(all_pairs[0]), cmp_KeyValue_bykey);
+
  cleanup:
   free_asc_file(lines, 0);
 
@@ -2851,40 +2848,71 @@ pipethrough(int infd, int outfd)
   return(0);  
 }
 
-Flag
-fsentry_modifyable(UChar * path)
+UChar *
+parentdir(UChar * dir, Flag resolve)
 {
-  UChar	*realpath = NULL, *cptr;
-  Flag	r = DONT_KNOW;
+  UChar		*cptr, *realpath;
 
+  cptr = realpath = mkabspath(dir, NULL);
+  if(!cptr)
+    return(NULL);
+
+  if(resolve){
+    realpath = resolve_path(cptr, NULL);
+    free(cptr);
+    if(!realpath)
+	return(NULL);
+  }
+
+  cleanpath__(realpath);
+
+  if(FN_ISROOTDIR(realpath))
+    return(realpath);
+
+  cptr = FN_LASTDIRDELIM(realpath);
+  if(!cptr){
+    free(realpath);
+    return(NULL);	/* This cannot happen */
+  }
+
+  if(cptr == (UChar *) FN_FIRSTDIRSEP(realpath))
+    cptr += strlen(FN_DIRSEPSTR);
+
+  *cptr = '\0';
+
+  return(realpath);
+}
+
+Flag
+fsentry_access(UChar * path, int flag)
+{
+  UChar		*pdir = NULL;
+  Flag		r = DONT_KNOW, resolve;
+  struct stat	statb;
+
+  if(!path){
+    errno = EINVAL;
+    return(NO);
+  }
   if(!strcmp(path, "")){
     errno = EINVAL;
     return(DONT_KNOW);
   }
 
-  cptr = mkabspath(path, NULL);
-  if(cptr){
-    realpath = resolve_path__(cptr, NULL);
-    free(cptr);
-  }
-  if(!realpath)
+  if(!access(path, flag))	/* access is ok -> end of story */
+    return(YES);
+
+  resolve = NO;
+  if(!lstat(path, &statb))
+    resolve = IS_SYMLINK(statb);
+
+  pdir = parentdir(path, resolve);
+  if(!pdir)
     return(DONT_KNOW);
 
-  forever{
-    if(!access(realpath, W_OK))
-	CLEANUPR(YES);
-    if(FN_ISROOTDIR(realpath))
-	CLEANUPR(NO);
-	
-    cptr = FN_LASTDIRDELIM(realpath);
-    if(cptr == (UChar *) FN_FIRSTDIRSEP(realpath))
-	cptr++;
+  r = (access(pdir, X_OK) ? DONT_KNOW : NO);
 
-    *cptr = '\0';
-  }
-
- cleanup:
-  free(realpath);
+  free(pdir);
   return(r);
 }
 
