@@ -1,8 +1,8 @@
 /****************** Start of $RCSfile: genutils.c,v $  ****************
 *
-* $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/genutils.c,v $
-* $Id: genutils.c,v 1.4 2004/07/08 20:34:45 alb Exp alb $
-* $Date: 2004/07/08 20:34:45 $
+* $Source: /home/alb/afbackup/afbackup-3.5.2.1/RCS/genutils.c,v $
+* $Id: genutils.c,v 1.5 2006/12/12 20:21:11 alb Exp alb $
+* $Date: 2006/12/12 20:21:11 $
 * $Author: alb $
 *
 *
@@ -15,7 +15,7 @@
 #include <conf.h>
 #include <version.h>
 
-  static char * fileversion = "$RCSfile: genutils.c,v $ $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/genutils.c,v $ $Id: genutils.c,v 1.4 2004/07/08 20:34:45 alb Exp alb $ " PACKAGE " " VERSION_STRING;
+  static char * fileversion = "$RCSfile: genutils.c,v $ $Source: /home/alb/afbackup/afbackup-3.5.2.1/RCS/genutils.c,v $ $Id: genutils.c,v 1.5 2006/12/12 20:21:11 alb Exp alb $ " PACKAGE " " VERSION_STRING;
 
 #include <stdio.h>
 #include <string.h>
@@ -128,6 +128,36 @@ nomemmsgexit(void * ptr, FILE * fp)
   }
 }
 
+static	UChar	*yesstrings[] = {
+			"1",
+			"yes",
+			"true",
+			"ok",
+			TN_("Yes"),
+			TN_("true"),
+			TN_("ok"),
+			NULL,
+		};
+
+Flag
+is_yes(UChar * string, Int32 len)
+{
+  UChar	**cpptr;
+  int	r;
+
+  for(cpptr = yesstrings; *cpptr; cpptr++){
+    if(len > 0)
+	r = strncasecmp(T_(*cpptr), string, len);
+    else
+	r = strcasecmp(T_(*cpptr), string);
+
+    if(!r)
+	return(YES);
+  }
+
+  return(NO);
+}
+
 /*
  * Returns a pointer to the first non-whitespace character in string.
  * Returns NULL and sets errno on error.
@@ -776,28 +806,37 @@ sscanword(
    UChar         *str,
    UChar         *word)
 {
-  Int32		i = 0, pos = 0;
+  UChar		*wptr;
 
-  while(ISSPACE(str[pos])){
-	if(str[pos] == '\0'){
+  wptr = word;
+
+  while(ISSPACE(*str)){
+	if(*str == '\0'){
 	    if(word)
-		word[0] = '\0';
+		*wptr = '\0';
 	    return(NULL);
 	}
-	pos++;
+	str++;
   }
 
-  while(!ISSPACE(str[pos])){
+  while(!ISSPACE(*str)){
 	if(word)
-	  word[i] = str[pos];
-	pos++;
-	i++;
+	  *wptr = *str;
+	str++;
+	wptr++;
   }
 
-  if(word)
-    word[i] = '\0';
+  if(word){
+    if(wptr == str && *str)
+ 	str++;
 
-  return(str + pos);
+    *wptr = '\0';
+  }
+
+  while(*str && isspace(*str))
+    str++;
+
+  return(str);
 }
 
 /*
@@ -847,8 +886,12 @@ sscanwordq(UChar * str, UChar * word)
     }
   }
 
-  if(word)
+  if(word){
+    if(word == str && *str)
+      str++;
+
     *word = '\0';
+  }
 
   return(first_nospace(str));	
 }
@@ -1330,6 +1373,11 @@ word_count(UChar * line)
 {
     Int32        i = 0, nr = 0;
 
+    if(!line){
+	errno = EINVAL;
+	return(-1);
+    }
+
     for (;;) {
 	while (ISSPACE(line[i])) {
 	    if (line[i] == '\n' || line[i] == '\0')
@@ -1844,15 +1892,21 @@ UChar *
 strapp(UChar * string1, UChar * string2)
 {
   UChar		*newstr;
+  Int32		len1, len2;
 
-  newstr = (UChar *) malloc_forced((strlen(string1) + strlen(string2) + 1)
-				* sizeof(UChar));
+  len1 = (string1 ? strlen(string1) : 0);
+  len2 = (string2 ? strlen(string2) : 0);
+
+  newstr = (UChar *) malloc_forced((len1 + len2 + 1) * sizeof(UChar));
 
   if(!newstr)
     return(NULL);
 
-  strcpy(newstr, string1);
-  strcat(newstr, string2);
+  strcpy(newstr, "");
+  if(string1)
+    strcat(newstr, string1);
+  if(string2)
+    strcat(newstr, string2);
 
   return(newstr);
 }
