@@ -1,8 +1,8 @@
 /****************** Start of $RCSfile: mserver.c,v $  ****************
 *
-* $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/mserver.c,v $
-* $Id: mserver.c,v 1.5 2005/01/15 08:56:22 alb Exp alb $
-* $Date: 2005/01/15 08:56:22 $
+* $Source: /home/alb/afbackup/afbackup-3.5.2.1/RCS/mserver.c,v $
+* $Id: mserver.c,v 1.8 2007/11/14 21:12:25 alb Exp alb $
+* $Date: 2007/11/14 21:12:25 $
 * $Author: alb $
 *
 *
@@ -15,7 +15,7 @@
 #include <conf.h>
 #include <version.h>
 
-  static char * fileversion = "$RCSfile: mserver.c,v $ $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/mserver.c,v $ $Id: mserver.c,v 1.5 2005/01/15 08:56:22 alb Exp alb $ " PACKAGE " " VERSION_STRING;
+  static char * fileversion = "$RCSfile: mserver.c,v $ $Source: /home/alb/afbackup/afbackup-3.5.2.1/RCS/mserver.c,v $ $Id: mserver.c,v 1.8 2007/11/14 21:12:25 alb Exp alb $ " PACKAGE " " VERSION_STRING;
 
 #include <stdio.h>
 #include <string.h>
@@ -151,6 +151,9 @@ UChar		*bindir = NULL;
 UChar		*libdir = NULL;
 UChar		*confdir = NULL;
 
+UChar		*pidfile = NULL;
+Flag		daemonized = NO;
+
 ReplSpec	replacements[] = {
 	{ "%B",	NULL, &bindir },
 	{ "%L",	NULL, &libdir },
@@ -185,7 +188,6 @@ ParamFileEntry	entries[] = {
 static Int32	handle_prot_func(int, void *, Int32, void *,
 					TcpMuxCallbDoneActions *, void *);
 
-
 static void
 do_exit(int est)
 {
@@ -202,6 +204,9 @@ do_exit(int est)
     est = WEXITSTATUS(pst);
   }
 
+  if(pidfile)
+    unlink(pidfile);
+
   exit(est);
 }
 
@@ -225,6 +230,13 @@ logmsg(int prio, UChar * fmt, ...)
   if(lfp)
     fclose(lfp);
 
+  if(lfp && daemonize){
+    va_start(args, fmt);
+    vfprintf(stderr, fmt, args);
+    fflush(stderr);
+    va_end(args);
+  }
+
   if(syslog_ident){
     va_start(args, fmt);
     gvsyslog(syslog_ident, LOG_CONS | LOG_PID, LOG_DAEMON, prio, fmt, args);
@@ -317,6 +329,12 @@ sig_handler(int sig)
     case SIGUSR1:
 	edebug = 0;
 	break;
+    case SIGTERM:
+    case SIGINT:
+    case SIGABRT:
+    case SIGQUIT:
+    case SIGHUP:
+	do_exit(128 + sig);
   }
 }
 
@@ -346,7 +364,7 @@ check_interrupted(ConnStatus * connstat)
 static void
 usage(UChar * prnam)
 {
-  logmsg(LOG_ERR, T_("Usage: %s [ -sbD ] [ <configuration-file> ] [ -l <logging-file> ] [ -L <locale> ]\n"), prnam);
+  logmsg(LOG_ERR, T_("Usage: %s [ -db ] [ <configuration-file> ] [ -p <service/port> ] [ -L <locale> ]\n"), prnam);
   do_exit(1);
 }
 
@@ -2218,6 +2236,11 @@ main(int argc, char ** argv)
   memcpy(nargv + 1 + i, argv + 1, sizeof(char *) * (argc - 1));
   nargv[argc + i] = NULL;
 
+  if(daemonize){
+    cptr = FN_BASENAME(progname);
+    EM__(pidfile = strchain(vardir, FN_DIRSEPSTR, cptr, ".pid", NULL));
+  }
+
   if(daemonize && ! edebug && ! restart){
     i = fork_forced();
     if(i < 0){
@@ -2230,15 +2253,41 @@ main(int argc, char ** argv)
 
 	detach_from_tty();
 
-	close(0);
-	close(1);
-	close(2);
+	daemonized = YES;
     }
     if(i > 0){
 	exit(0);
     }
+
+    i = set_wlock(pidfile);	/* lock here and not in parent, because */
+    if(i < 0){			/* locks are not inherited by child procs */
+	fprintf(stderr,
+		T_("Error: Cannot lock PID file `%s', probably `%s' is already running.\n"),
+		pidfile, cptr);
+	ZFREE(pidfile);
+	do_exit(12);
+    }
+
+    sprintf(buf, "%d\n", getpid());
+    k = strlen(buf);
+    if(write_forced(i, buf, k) < k
+#ifdef	HAVE_FSYNC
+		|| fsync(i) < 0
+#endif
+				){
+	fprintf(stderr, T_("Warning: Cannot write PID file `%s' correctly.\n"),
+		pidfile);
+	unlink(pidfile);
+	ZFREE(pidfile);
+    }
   }
 
+  signal(SIGTERM, sig_handler);
+  signal(SIGINT, sig_handler);
+  signal(SIGABRT, sig_handler);
+  signal(SIGQUIT, sig_handler);
+  signal(SIGHUP, sig_handler);
+
   if(!restart){
     cptr = loggingfile;
     if(!cptr)
@@ -2289,6 +2338,12 @@ main(int argc, char ** argv)
     }
   }
 
+  if(daemonized){
+    close(0);
+    close(1);
+    close(2);
+  }
+
   if(!daemonize)
     i = start_slave_server();
 
@@ -2301,25 +2356,33 @@ main(int argc, char ** argv)
   i = tcp_mux_service(portnum, NULL,
 		init_conn_func, handle_prot_func, close_conn_func,
 		maxconns, toomany_func, tcpmux_flags, NULL);
-  if(i)
+  if(i){
+    i = errno;
     logmsg(LOG_ERR, T_("Warning: Service unexpectedly terminated.\n"));
-
-  buf[0] = GOODBYE;
-  k = (write_forced(tofd, buf, 1) != 1);
-  if(k){
-    logmsg(LOG_ERR, T_("Warning: Cannot send command %d (%c) to slave server.\n"),
-			buf[0], buf[0]);
-    i = i ? i : (k ? - errno : 0);
+    if(i == EADDRINUSE){
+	logmsg(LOG_ERR, T_("Warning: Address already in use. Exiting.\n"));
+	daemonize = NO;
+    }
   }
 
-  k = result(fromfd, NULL);
-  if(k){
-    logmsg(LOG_ERR, T_("Warning: Did not get success response for command %d (%c) from slave server.\n"),
+  if(pid > 0){
+    buf[0] = GOODBYE;
+    k = (write_forced(tofd, buf, 1) != 1);
+    if(k){
+	logmsg(LOG_ERR, T_("Warning: Cannot send command %d (%c) to slave server.\n"),
 			buf[0], buf[0]);
-    i = i ? i : k;
-  }
+	i = i ? i : (k ? - errno : 0);
+    }
 
-  waitpid_forced(pid, &pst, 0);
+    k = result(fromfd, NULL);
+    if(k){
+	logmsg(LOG_ERR, T_("Warning: Did not get success response for command %d (%c) from slave server.\n"),
+			buf[0], buf[0]);
+	i = i ? i : k;
+    }
+
+    waitpid_forced(pid, &pst, 0);
+  }
 
   close(fromfd);
   close(tofd);
