@@ -1,8 +1,8 @@
 /****************** Start of $RCSfile: mvals.h,v $  ****************
 *
-* $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/mvals.h,v $
-* $Id: mvals.h,v 1.3 2004/07/08 20:34:45 alb Exp alb $
-* $Date: 2004/07/08 20:34:45 $
+* $Source: /home/alb/afbackup/afbackup-3.5.2.1/RCS/mvals.h,v $
+* $Id: mvals.h,v 1.4 2006/12/12 20:21:11 alb Exp alb $
+* $Date: 2006/12/12 20:21:11 $
 * $Author: alb $
 *
 *
@@ -165,4 +165,9 @@ extern "C" {
 #define	MINReal64	MINDOUBLE
 #define	MINReal32	MINFLOAT
 
+/* finally hand crafted avoiding integer overflow or similar warnings */
+#ifndef	MAXINT
+#define	MAXINT	((((1 << (sizeof(int) * 8 - 2)) - 1) << 1) | 1)
+#endif
+
 #endif	/* _MVALS_H */
