@@ -1,8 +1,8 @@
 /****************** Start of $RCSfile: pack.c,v $  ****************
 *
-* $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/pack.c,v $
-* $Id: pack.c,v 1.6 2005/07/31 15:42:10 alb Exp alb $
-* $Date: 2005/07/31 15:42:10 $
+* $Source: /home/alb/afbackup/afbackup-3.5.2.1/RCS/pack.c,v $
+* $Id: pack.c,v 1.8 2007/01/04 20:50:20 alb Exp alb $
+* $Date: 2007/01/04 20:50:20 $
 * $Author: alb $
 *
 *
@@ -15,7 +15,7 @@
 #include <conf.h>
 #include <version.h>
 
-  static char * fileversion = "$RCSfile: pack.c,v $ $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/pack.c,v $ $Id: pack.c,v 1.6 2005/07/31 15:42:10 alb Exp alb $ " PACKAGE " " VERSION_STRING;
+  static char * fileversion = "$RCSfile: pack.c,v $ $Source: /home/alb/afbackup/afbackup-3.5.2.1/RCS/pack.c,v $ $Id: pack.c,v 1.8 2007/01/04 20:50:20 alb Exp alb $ " PACKAGE " " VERSION_STRING;
 
 #include <stdio.h>
 #include <string.h>
@@ -626,35 +626,68 @@ write_acls(UChar * name, struct stat * statb, AarParams * params)
 #else	/* defined(USE_POSIX_ACLS) */
 
 #ifdef	USE_SOLARIS2_ACLS
+#ifdef	USE_NFS4_ACLS
+
+/*if(!acl_trivial(name))*/
+/* buggy, return value is inconsistent with acl_get(..., ACL_NO_TRIVIAL, ...) */
+#endif
 {
-  aclent_t	*acls;
+  aclent_t	*pacls;
   int		nacls;
 
   nacls = acl(name, GETACLCNT, 0, NULL);
   if(nacls > MIN_ACL_ENTRIES){
-    acls = NEWP(aclent_t, nacls);
-    if(!acls)
+    pacls = NEWP(aclent_t, nacls);
+    if(!pacls)
 	return(-ENOMEM);
 
-    nacls = acl(name, GETACL, nacls, acls);
+    nacls = acl(name, GETACL, nacls, pacls);
 
     sprintf(sbuf, "%d;%d;", (int) SOLARIS2_ACL, (int) nacls);
     if( (k = params->outputfunc(sbuf, strlen(sbuf), params)) )
 	return(k);
 
     for(i = 0; i < nacls; i++){
-	sprintf(sbuf, "%d;%d;%d;", (int) acls[i].a_type,
-			(int) acls[i].a_id, (int) acls[i].a_perm);
+	sprintf(sbuf, "%d;%d;%d;", (int) pacls[i].a_type,
+			(int) pacls[i].a_id, (int) pacls[i].a_perm);
 	if( (k = params->outputfunc(sbuf, strlen(sbuf), params)) ){
-	  free(acls);
+	  free(pacls);
 	  return(k);
 	}
     }
 
-    free(acls);
+    free(pacls);
     if( (k = params->outputfunc(".", 1, params)) )
 	return(k);
   }
+#ifdef	USE_NFS4_ACLS
+  else{
+    acl_t	*acls = NULL;
+    char	*aclstr = NULL;
+
+    if(!acl_get(name, ACL_NO_TRIVIAL, &acls)){
+      if(acls){
+	aclstr = acl_totext(acls, ACL_COMPACT_FMT | ACL_APPEND_ID);
+	if(!aclstr)
+	  return(-ENOMEM);
+
+	sprintf(sbuf, "%d;%d;", (int) NFS4_ACL, strlen(aclstr));
+	if( (k = params->outputfunc(sbuf, strlen(sbuf), params)) ){
+	  free(aclstr);
+	  return(k);
+	}
+	if( (k = params->outputfunc(aclstr, strlen(aclstr), params)) ){
+	  free(aclstr);
+	  return(k);
+	}
+
+	free(aclstr);
+	if( (k = params->outputfunc(".", 1, params)) )
+	  return(k);
+      }
+    }
+  }
+#endif
 }
 #endif
 
