@@ -1,8 +1,8 @@
 /****************** Start of $RCSfile: sysutils.c,v $  ****************
 *
-* $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/sysutils.c,v $
-* $Id: sysutils.c,v 1.8 2005/07/31 15:42:11 alb Exp alb $
-* $Date: 2005/07/31 15:42:11 $
+* $Source: /home/alb/afbackup/afbackup-3.5.2.1/RCS/sysutils.c,v $
+* $Id: sysutils.c,v 1.10 2007/01/04 20:50:27 alb Exp alb $
+* $Date: 2007/01/04 20:50:27 $
 * $Author: alb $
 *
 *
@@ -15,7 +15,7 @@
 #include <conf.h>
 #include <version.h>
 
-  static char * fileversion = "$RCSfile: sysutils.c,v $ $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/sysutils.c,v $ $Id: sysutils.c,v 1.8 2005/07/31 15:42:11 alb Exp alb $ " PACKAGE " " VERSION_STRING;
+  static char * fileversion = "$RCSfile: sysutils.c,v $ $Source: /home/alb/afbackup/afbackup-3.5.2.1/RCS/sysutils.c,v $ $Id: sysutils.c,v 1.10 2007/01/04 20:50:27 alb Exp alb $ " PACKAGE " " VERSION_STRING;
 
 #include <stdio.h>
 #include <unistd.h>
@@ -1659,7 +1659,7 @@ get_fs_status(
 #ifdef	HAVE_STATVFS		/* sees to be well-defined */
 	  if(statvfs(path, &statfsb) == -1)
 #else
-#if defined(_AIX) || defined(linux) || defined(__hpux) || defined(hpux) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__)
+#if defined(_AIX) || defined(linux) || defined(__hpux) || defined(hpux) || defined(__FreeBSD__) || ( defined(__GLIBC__) && defined(__FreeBSD_kernel__) ) || defined(__OpenBSD__) || defined(__NetBSD__)
 	  if(statfs(path, &statfsb) == -1)
 #else
 #if defined(sun)
@@ -1859,13 +1859,13 @@ cmp_mnt_by_devno(void * ptr1, void * ptr2)
 UChar *
 mount_tab_file()
 {
-#if	defined(linux) || defined(sgi) || ( defined(sun) && ! defined(HAVE_GETMNTENT_TWO_ARGS) )
+#if	defined(linux) || defined(sgi) || ( defined(sun) && ! defined(HAVE_GETMNTENT_TWO_ARGS) ) || ( defined(__GLIBC__) && defined(__FreeBSD_kernel__) )
 #define	MTABFILE	"/etc/mtab"
 #else
 #if	defined(sun) || defined(__hpux) || defined(hpux) || defined(UnixWare_5)
 #define	MTABFILE	"/etc/mnttab"
 #else
-#if	defined(__osf__) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(_AIX)
+#if	defined(__osf__) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(_AIX) || defined(__APPLE__)
 #define	MTABFILE	NULL
 #endif
 #endif
