@@ -1,8 +1,8 @@
 /****************** Start of $RCSfile: vars.c,v $  ****************
 *
-* $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/vars.c,v $
-* $Id: vars.c,v 1.3 2005/01/15 08:54:35 alb Exp alb $
-* $Date: 2005/01/15 08:54:35 $
+* $Source: /home/alb/afbackup/afbackup-3.5.2.1/RCS/vars.c,v $
+* $Id: vars.c,v 1.4 2006/12/12 20:21:19 alb Exp alb $
+* $Date: 2006/12/12 20:21:19 $
 * $Author: alb $
 *
 *
@@ -15,7 +15,7 @@
 #include <conf.h>
 #include <version.h>
 
-  static char * fileversion = "$RCSfile: vars.c,v $ $Source: /home/alb/afbackup/afbackup-3.3.9beta4/RCS/vars.c,v $ $Id: vars.c,v 1.3 2005/01/15 08:54:35 alb Exp alb $ " PACKAGE " " VERSION_STRING;
+  static char * fileversion = "$RCSfile: vars.c,v $ $Source: /home/alb/afbackup/afbackup-3.5.2.1/RCS/vars.c,v $ $Id: vars.c,v 1.4 2006/12/12 20:21:19 alb Exp alb $ " PACKAGE " " VERSION_STRING;
 
 #include <string.h>
 #include <stdarg.h>
@@ -28,12 +28,23 @@
 #define	CLEANUP	{ goto cleanup; }
 
 
-static Int32
-cmp_KeyValue(void * ptr1, void * ptr2)
+Int32
+cmp_KeyValue_bykey(void * ptr1, void * ptr2)
 {
   return(strcmp(((KeyValue *) ptr1)->key, ((KeyValue *) ptr2)->key));
 }
 
+Int32
+cmp_KeyValue_bykeyn(void * ptr1, void * ptr2)
+{
+  int		k1, k2;
+
+  k1 = k2 = 0;
+  sscanf(((KeyValue *) ptr1)->key, "%d", &k1);
+  sscanf(((KeyValue *) ptr2)->key, "%d", &k2);
+  return(k1 > k2 ? 1 : (k1 < k2 ? -1 : 0));
+}
+
 Int32
 set_var(KeyValue ** vars, UChar * name, UChar * value)
 {
@@ -48,7 +59,7 @@ set_var(KeyValue ** vars, UChar * name, UChar * value)
     newvars = *vars;
     for(vptr = newvars, n = 0; vptr->key; vptr++, n++);
 
-    vptr = b_locate(&cmpvar, newvars, n, sizeof(KeyValue), cmp_KeyValue);
+    vptr = b_locate(&cmpvar, newvars, n, sizeof(KeyValue), cmp_KeyValue_bykey);
     voffs = vptr - newvars;
 
     if(!vptr->key)
@@ -121,7 +132,7 @@ unset_var(KeyValue * vars, UChar * name)
   cmpvar.key = name;
   for(n = 0, vptr = vars; vptr->key; vptr++, n++);
 
-  vptr = b_search(&cmpvar, vars, n, sizeof(KeyValue), cmp_KeyValue);
+  vptr = b_search(&cmpvar, vars, n, sizeof(KeyValue), cmp_KeyValue_bykey);
 
   if(vptr){
     pos = vptr - vars;
@@ -146,7 +157,7 @@ get_var(KeyValue * vars, UChar * name, Flag all)
   cmpvar.key = name;
   for(n = 0, vptr = vars; vptr->key; vptr++, n++);
 
-  vptr = b_search(&cmpvar, vars, n, sizeof(KeyValue), cmp_KeyValue);
+  vptr = b_search(&cmpvar, vars, n, sizeof(KeyValue), cmp_KeyValue_bykey);
   if(vptr){
     rptr = vptr->value;
     if(all)
@@ -278,7 +289,7 @@ set_named_data(KeyValue ** vars, UChar * name, void * value, size_t size)
     newvars = *vars;
     n = num_named_data(*vars);
 
-    vptr = b_locate(&cmpvar, newvars, n, sizeof(KeyValue), cmp_KeyValue);
+    vptr = b_locate(&cmpvar, newvars, n, sizeof(KeyValue), cmp_KeyValue_bykey);
     voffs = vptr - newvars;
 
     if(!vptr->key)
@@ -349,7 +360,7 @@ get_named_data(KeyValue * data, UChar * name, Flag all, size_t * size)
 
   cmpvar.key = name;
   vptr = b_search(&cmpvar, data, num_named_data(data),
-				sizeof(KeyValue), cmp_KeyValue);
+				sizeof(KeyValue), cmp_KeyValue_bykey);
   if(!vptr)
     return(NULL);
   rptr = vptr->value;
